c  Version 9 June 1997
c
c  Computes transfer function and multiple coherence between up to
c  three output and three input data series using robust, controlled
c  leverage, two stage least squares method.
c
c  Computation is by section averaging using variable section lengths.
c  Robust weights are computed using a Rayleigh distribution model on the
c  regression residuals and applied equally to the real and imaginary parts.
c  Leverage weights are computed by comparing the hat matrix diagonal elements 
c  to their expected value. Optional coherence thresholding can be applied with
c  a combination of a selected coherence value and/or a fractional trim value.
c  Confidence limits are obtained using the unbalanced jackknife.
c
c
c  npcsm = maximum number of data pieces
c  nptsm = maximum number of data
c  nptssm = maximum fft section length
c  noutm = maximum number of output time series
c  ninpm = maximum number of input time series
c  nrefm = maximum number of reference time series
c  nrsitem = maximum number of reference sites
c  nsectm = maximum number of data sections
c  nblkmin = minimum number of data per block for coherence thresholding
c  nfsm = maximum number of frequencies to be used for a given section size
c  nfm = maximum number of output frequencies
c  narm = maximum ar filter length
c  nerr = output unit for diagnostic file
c  inter = logical flag for interactive (.true.) or GUI (.false.) version
c
      include 'parameters.h'
      character filnam*80,form*1,ofil*70,rfout(2)*1,postfix*30,
     $          cn*2,star*3,cpar(nserm)*80
      double precision var,wtsum,dpss,dwork,dofscal,evalu
      complex ffts,ftsec,tb,hn,hr,hd1,gn,gr,fft,ffts1,resq,oresq
      double complex b,aa,bb,ob
      logical compare,c2flag,c2flag1,convflag,cflag1,cflag2,l1,
     $        l2
      dimension nstart(npcsm+1),npts(npcsm)
      dimension frej(nfm),data(nptsm,nserm),
     $          fpar(nfm,nserm),arf(narm+1,nserm,npcsm),
     $          wsave(4*nptssm+15),ffts1(nsectm,nfsm),
     $          ffts(nsectm,nserm,nfsm),fft(nptssm),
     $          ftsec(nsectm,ninpm+nrefm+1),
     $          tb(nrefm,ninpm,nfm),res(nsectm),pwro(nsectm),
     $          pwri(nsectm),tag(nsectm),b(nrefm),aa(ninpm,ninpm),
     $          bb(ninpm),hn(noutm,ninpm,nfm),
     $          hd1(ninpm,nsectm),hat(nsectm),l1(nsectm),
     $          l2(nsectm),varn(noutm,ninpm,nfm),temp(nsectm),
     $          gn(noutm,nfm),dfn(noutm,nfm),gd1(nsectm),
     $          gnlo(noutm,nfm),gnhi(noutm,nfm),qq(nsectm),
     $          xindex(nsectm),yindex(nsectm),w(nsectm),
     $          what(nsectm),wt(nsectm),hr(noutm,ninpm,nfm),
     $          varr(noutm,ninpm,nfm),gr(noutm,nfm),
     $          dfr(noutm,nfm),grlo(noutm,nfm),
     $          grhi(noutm,nfm),freq(nfm),ob(nrefm)
      dimension fracdat(noutm,nfm),arft(narm+1),theta1(nrsitem+2),
     $          theta2(nrsitem+2),phi(nrsitem+2),dpss(nptssm),
     $          dwork(19*nptssm/4+12),c2b(nsectm,ninpm),
     $          c2flag(nsectm),c2e(nsectm,noutm),
     $          c2flag1(nsectm),fracthresh(noutm,nfm),
     $          wt1(nsectm),wt2(nsectm),convflag(noutm,nfm)
      double precision pi,rad,tpi
      parameter (pi=3.141592653589793d0,rad=pi/180.d0,tpi=2.d0*pi)
      data star/'***'/,c2cut/0./,c2threshe/0./,c2threshe1/0./,
     $     c2threshb/0./,c2trime/0./,c2trimb/0./,ctrime1/0./,
     $     nc2blk/10/
      data eps/0.01/,eps1/0.1/,wtthresh/0.5/
      equivalence (ffts1,ftsec),(dwork,wsave)
c
c  input run parameters
c
      if(inter)write(6,6)
      read(5,*)nout
      if(inter)write(6,7)
      read(5,*)ninp
      if(inter)write(6,8)
      read(5,*)nref
      if(nref.gt.3)then
        nrr=1
        if(inter)write(6,9)
        read(5,*)nr3,nr2
      elseif(nref.eq.3)then
        nrr=1
        nr3=1
        nr2=0
      elseif(nref.eq.2)then
        nr3=0
        nr2=1
      endif
      nser=nout+ninp+nref
      if(nref.lt.3)then
        if(inter)write(6,10)
        read(5,*)nrr
      endif
    1 if(inter)write(6,11)
      read(5,*)tbw
      if(inter)then
        if((tbw.lt.1.).or.(tbw.gt.4.))then
          write(6,*)' time-bandwidth out of bounds-should be 1 to 4'
          goto 1
        endif
      endif
      offs=0.57-0.07*tbw
    2 if(inter)write(6,12)
      read(5,*)nfft,nsctinc,nsctmax
      if(inter)write(6,13)
      read(5,*)nf1,nfsect
      if(inter)then
        if(nfft/(float(nsctinc**(nsctmax-1))*offs).gt.nsectm)then
          write(6,*)' number of sections exceeds dimension of',nsectm
          goto 2
        endif
        if(nfft.gt.nptssm)then
          write(6,*)' fft length exceeds dimension of',nptssm
          goto 2
        endif
        if(nsctmax*nfsect.gt.nfm)then
          write(6,*)' number of frequencies exceeds dimension of',nfm
          goto 2
        endif
        if(nfsect.gt.nfsm)then
          write(6,*)' no. of freqs per sect exceeds dimension of',nfsm
          goto 2
        endif
      endif
      if(inter)write(6,14)
      read(5,*)uin,ainlin,ainuin
      if(nrr.eq.1)then
        if(inter)write(6,15)
        read(5,*)c2threshb,c2trimb
        c2threshb=min(c2threshb,1.)
      endif
      if(inter)write(6,16)
      read(5,*)c2threshe,c2trime
      c2threshe=min(c2threshe,1.)
      if(nout.eq.3)then
        if(inter)write(6,17)
        read(5,*)nz
        if(nz.eq.0)then
          if(inter)write(6,16)
          read(5,*)c2threshe1,c2trime1
        endif
      else
        nz=0
      endif
      if((c2threshb.ne.0.).or.(c2threshe.ne.0.).or.
     $   (c2threshe1.ne.0))then
        if(inter)write(6,18)
        read(5,*)nc2blk
        if(nc2blk.lt.nblkmin)then
          if(inter)write(6,*)' block size increased to',nblkmin
          nc2blk=nblkmin
        endif
        c2crit=1.-0.05**(2./(2.*(nc2blk-ninp)))
        if((inter).and.(c2threshb.ne.0.).and.(c2threshb.lt.c2crit))then
          write(6,*)' zero coherence at 95% significance of',c2crit
          write(6,*)' is larger than specified value of ',c2threshb
        endif
        if((inter).and.(c2threshe.ne.0.).and.(c2threshe.lt.c2crit))then
          write(6,*)' zero coherence at 95% significance of',c2crit
          write(6,*)' is larger than specified value of ',c2threshe
        endif
        if((inter).and.(c2threshe1.ne.0.).and.
     $     (c2threshe1.lt.c2crit))then
          write(6,*)' zero coherence at 95% significance of',c2crit
          write(6,*)' is larger than specified value of ',c2threshe1
        endif
        if(inter)write(6,19)
        read(5,*)c2lo,c2hi
      else
        nc2blk=10
        c2threshb=0.
        c2threshe=0.
        c2threshe1=0.
      endif
      if(inter)write(6,20)
      read(5,21)ofil
      if(inter)write(6,22)
      read(5,*)nlev
      if(nlev.ge.1)then
        if(inter)write(6,23)
        read(5,24)rfout(1),rfout(2)
        if(rfout(1).eq.'Y')rfout(1)='y'
        if(rfout(2).eq.'Y')rfout(2)='y'
      endif
      if(inter)write(6,25)
      read(5,*)nfrej
      if(nfrej.ne.0)then
        if(inter)write(6,26)
        do 5 i=1,nfrej
    5     read(5,*)frej(i)
      endif
      if(inter)write(6,27)
      read(5,*)nfil
c
c  open diagnostic file
c
      open (unit=nerr,file=ofil(1:lenb(ofil))//'.diag',
     $      status='unknown')
      if(nfil.gt.nfm)then
        if(inter)then
          write(6,*)' number of filter coefficients exceeds dimension'
        else
          write(nerr,*)
     $              ' number of filter coefficients exceeds dimension'
        endif 
        stop
      endif
    6 format(' input the number of output time series:',$)
    7 format(' input the number of input time series:',$)
    8 format(' input the number of reference time series:',$)
    9 format(' input the number of 3/2 component reference sites:',$)
   10 format(' select r-r (0) or 2-stage robust method (1):',$)
   11 format(' select tbw for prolate data window (1 to 4):',$)
   12 format(' input the initial section length, divisor,'
     $       ' and maximum number of sections:',$)
   13 format(' input the starting index and number of',
     $       ' frequencies per section:',$)
   14 format(' input the robustness and leverage parameters',
     $       ' (-999 for none):',$)
   15 format(' input the 1st stage coherence threshold and',
     $       ' trim value:',$)
   16 format(' input the 2nd stage coherence threshold and',
     $       ' trim value:',$)
   17 format(' input the 3rd variable threshold mode:',$)
   18 format(' input the coherence threshold block size:',$)
   19 format(' input the lower and upper frequencies:',$)
   20 format(' input the output filename root:',$)
   21 format(a)
   22 format(' input the output level (0-4):',$)
   23 format(' save delete-1 response functions (y or n)?')
   24 format(a1,x,a1)
   25 format(' input the number of rejection frequencies:',$)
   26 format(' input the frequencies')
   27 format(' input the number of filter parameters',
     $       ' (0 for filename):',$)
c
c  input data
c
      if(inter)write(6,40)
      read(5,*)npcs
      if(inter)write(6,41)
      read(5,*)nar
      nstart(1)=1
      nptstot=0
      do 35 n=1,nser
        if(inter)write(6,42)
        read(5,21)form
        if(form.eq.'f')form='F'
        if(form.eq.'u')form='U'
        if(form.eq.'U')then
          if(inter)write(6,43)
          read(5,*)nblock
        else
          nblock=0
        endif
        if(nfil.gt.0)then
          if(inter)write(6,44)
          read(5,*)(fpar(i,n),i=1,nfil)
        elseif(nfil.lt.0)then
	  if(inter)write(6,45)
	  read(5,21)cpar(n)
	endif
        do 34 nn=1,npcs
          if(inter)write(6,46)
          read(5,21)filnam
          if(inter)write(6,47)
          read(5,*)nskip,nread
          call input(filnam,form,nblock,nskip,nread,                 
     $               data(nstart(nn),n))
          if(n.eq.1)then
            nptstot=nptstot+nread
            npts(nn)=nread
            nstart(nn+1)=nstart(nn)+nread
            if(nptstot.gt.nptsm)then
              if(inter)then
                write(6,*)' number of data exceeds dimension of',
     $                    nptsm
              else
                write(nerr,*)' number of data exceeds dimension of',
     $                       nptsm
              endif
              stop
            endif
          elseif(nread.ne.npts(nn))then
            if(inter)then
              write(6,*)' number of data incorrect at series',n,
     $                  ' piece',nn
            else
              write(nerr,*)' number of data incorrect at series',n,
     $                  ' piece',nn
            endif
            stop
          endif
c
c  force data to have zero mean for each piece
c
          sum=0.d0
          do 30 i=nstart(nn),nstart(nn)+nread-1
   30       sum=sum+data(i,n)
          sum=sum/nread
          do 31 i=nstart(nn),nstart(nn)+nread-1
   31       data(i,n)=data(i,n)-sum 
   34     continue
   35   continue
      if(inter)write(6,48)
      if(nout.ge.2)read(5,*)theta1(1),theta2(1),phi(1)
      read(5,*)theta1(2),theta2(2),phi(2)
      do 36 i=1,nr3+nr2
   36   read(5,*)theta1(2+i),theta2(2+i),phi(2+i)
c
   40 format(' input the number of data pieces:',$)
   41 format(' input the length of the ar filter (0 for none):',$)
   42 format(' is the data formatted (f) or unformatted (u)?:',$)
   43 format(' input the block size:',$)
   44 format(' input the filter parameters')
   45 format(' input the filter character string')
   46 format(' input the filename:',$)
   47 format(' input the number of points to skip and read:',$)
   48 format(' input the rotation angles (cw=+)')
c
c  open delete-one files
c
      do 50 nn=1,nout
        if((nlev.ge.1).and.(rfout(1).eq.'y'))then
          filnam=ofil(1:lenb(ofil))//'.'//char(48+nn)//'n.d1rf'
          open (unit=22+nn,file=filnam,form='unformatted',
     $          status='unknown')
        endif
        if((nlev.ge.1).and.(rfout(2).eq.'y'))then
          filnam=ofil(1:lenb(ofil))//'.'//char(48+nn)//'r.d1rf'
          open (unit=32+nn,file=filnam,form='unformatted',
     $          status='unknown')
        endif
   50   continue
c
c  prewhiten data series with ar filter
c
      if(nar.gt.0)then
        do 60 jj=1,npcs
          n1=nstart(jj)
          do 60 j=1,nser
            n=npts(jj)
            call arfilt(n,nar,data(n1,j),wsave,arft,info)
            if((inter).and.(info.eq.1))then
              write(6,*)' WARNING-ill-conditioned Toeplitz matrix in'
              write(6,*)' call to arfilt series=',j,' piece=',jj
            endif
            do 55 i=1,nar+1
   55         arf(i,j,jj)=arft(i)
   60       if(j.eq.nser)npts(jj)=n
      endif
c
c  begin outer loop pivoting on section length
c
      nfl=1
      do 70 n=1,nout
	do 70 j=1,nfm
   70     convflag(n,j)=.true.
      do 790 nsc=1,nsctmax
        write(nerr,*)'*******************************************'
        if(mod(nfft,2).ne.0)nfft=nfft+1
        write(nerr,*)' section length=',nfft
        xnfft=nfft
        noff=offs*xnfft
c
c  initialize fft computation and compute slepian sequence data window
c
        call rtpss(nptssm,nfft,1,dble(tbw/xnfft),dpss,
     $             evalu,info,dwork)
        if((inter).and.(ierr.ne.0))
     $     write(6,*)' error in call to rtpss info=',ierr
        call cffti(nfft,wsave)
c
c  compute section ffts and correct for instrument response and prewhitening
c
        nsect=0
        do 90 jj=1,npcs
          if(npts(jj).lt.nfft)goto 90
          xnpts=npts(jj)
          nsct=(xnpts/xnfft+offs-1.)/offs
          ko=nstart(jj)
          do 80 j=1,nser
            call dataft(nsct,nf1,nfsect,nfft,noff,nfil,nar,
     $                  dpss,wsave,data(ko,j),fpar(1,j),
     $                  cpar(j),arf(1,j,jj),fft,ffts1)
            do 80 ii=1,nfsect
              do 80 i=1,nsct
   80           ffts(nsect+i,j,ii)=ffts1(i,ii)
          nsect=nsect+nsct
   90     continue
        nsect=(nsect/nc2blk)*nc2blk
        if(nsect.gt.nsectm)then
          if(inter)then
            write(6,*)' number of sections ',nsect,
     $                ' is larger than maximum dimension of ',nsectm
          else
            write(nerr,*)' number of sections ',nsect,
     $                   ' is larger than maximum dimension of ',nsectm
          endif
          stop
        elseif(nsect.eq.0)then
          goto 790
        endif
        xnsect=nsect
c
c  rotate ffts to new coordinate system
c  theta1 is angle of first component pos cw wrt geomagnetic north
c  theta2 is same for second component
c  phi is pos cw rotation angle of geomagnetic coordinate system
c  if original data are collected in orthogonal coordinate
c  system, set theta1=0, theta2=90
c
        if(nout.gt.1)then
c  first two ffts are assumed to be x and y and are rotated
c  if nout=3, third series is assumed to be z and is not rotated
          do 100 i=1,nfsect
            call rotate(nsect,theta1(1),theta2(1),phi(1),
     $                  ffts(1,1,i),ffts(1,2,i))
  100       continue
        endif
c
c  always rotate input ffts
c  if ninp=3, third series is assumed to be z and is not rotated
c
        if(ninp.gt.1)then
          do 102 i=1,nfsect
            call rotate(nsect,theta1(2),theta2(2),phi(2),
     $                  ffts(1,nout+1,i),ffts(1,nout+2,i))
  102       continue
        endif
c
c  always rotate remote reference ffts if nref.gt.1
c
        if(nref.gt.1)then
          do 104 j=1,nr3
            do 104 i=1,nfsect
              call rotate(nsect,theta1(2+j),theta2(2+j),phi(2+j),
     $                    ffts(1,nout+ninp+3*j-2,i),
     $                    ffts(1,nout+ninp+3*j-1,i))
  104         continue
          do 105 j=1,nr2
            do 105 i=1,nfsect
              call rotate(nsect,theta1(2+nr3+j),theta2(2+nr3+j),
     $                    phi(2+nr3+j),ffts(1,nout+ninp+3*nr3+2*j-1,i),
     $                    ffts(1,nout+ninp+3*nr3+2*j,i))
  105         continue
        endif
c
c  compute degrees of freedom inflation factor
c
        dofscal=1.
        nm=max(0,min(nfft,(nptstot-nfft)/(nsect-1)))
        do 110 i=1,nsect-1
          sum=0.d0
          do 109 j=1,nfft
            if(j+i*nm.le.nfft)sum=sum+dpss(j)*dpss(j+i*nm)
  109       continue
  110     dofscal=dofscal+2.*(1-float(i/nsect))*sum**2
        dofscal=1./dofscal
c
c  begin second loop pivoting on frequency
c
        do 780 nf=1,nfsect
          frq=float(nf+nf1-1)/xnfft
          do 115 k=1,nfrej
            if(compare(frq,frej(k),4))goto 780
  115       continue
          freq(nfl)=frq
          write(nerr,*)' $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$'
          write(nerr,*)' frequency=',freq(nfl)
c
c  begin first stage
c
          do 118 i=1,nsect/nc2blk
            c2flag(i)=.true.
	    do 118 j=1,ninp
  118         c2b(i,j)=1.0
	  do 119 i=1,nsect
	      wt1(i)=1.0
  119         wt2(i)=1.0
c
c  compute projected local magnetic field if required
c
          if(nrr.eq.0)goto 300
c
c coherence estimation by block averaging
c
          if(nlev.ge.2)then
            call bcoher2(nsect,ninp,ninp,nref,0,nc2blk,c2threshb,
     $                   c2trimb,ffts(1,nout+1,nf),xindex,c2flag,
     $                   c2cut,c2b,info)
            if(info.ne.0)then
              write(nerr,*)
     $          ' WARNING-singular matrix detected in',
     $          ' stage 1 coherence threshold at nsc=',nsc,' nf=',nf
            endif
          endif
          write(nerr,*)' stage 1 coherence threshold=',c2cut
c
c  begin third loop pivoting on input variable
c
	  do 265 nn=1,ninp
            write(nerr,*)' ++++++++++++++++++++++++++++++'
            write(nerr,*)' input series=',nn
c
c  perform coherence thresholding
c
            call threshold(nsect,nc2blk,ninp-nn+1,nref,c2flag,wt1,
     $                     wtthresh,ffts(1,nout+nn,nf),nsct,tag,pwro,
     $                     pwri,ftsec)
            if(c2cut.gt.0.)
     $        write(nerr,*)nsect-nsct,' sections discarded'
            write(nerr,*)' initial number of data=',nsct
            if(nsct.eq.0)goto 780
            xnsct=nsct
c
c  compute nonrobust transfer function
c
            do 125 i=1,nsct
  125         wt(i)=1.
            call transfer(nsct,nref,wt,ftsec,b,info)
            if(info.eq.1)then
              write(nerr,*)' WARNING-singular matrix detected in',
     $                     ' nonrobust magnetic transfer section'
            endif
            write(nerr,*)
     $       ' initial transfer function indexed by reference variable'
            do 130 i=1,nref
              tb(i,nn,nfl)=b(i)
  130         write(nerr,*)i,tb(i,nn,nfl)
            if(uin.eq.-999.)goto 265
c  compute hat matrix diagonal
c
            call hatdiag(nsct,nref,ftsec(1,2),wt,hat,info)
            if(info.ne.0)then
              write(nerr,*)
     $              ' WARNING-pivot block singular in nonrobust',
     $              ' magnetic transfer call to hatdiag info=',info
            endif
c
c  compute residuals
c
            call residual(nsct,nref,ftsec,b,wt,nw,res,oresq)
c
c  compute initial robust scale estimate using nonrobust residuals
c
	    do 132 i=1,nsct
  132         temp(i)=res(i)
            call mad(nsct,temp,scale)
c
c begin robust transfer function computation
c
c
c  huber weight regression section
c
            do 155 i=1,nsct
  155         xindex(i)=i
c
c  compute number of hat matrix iterations
c
	    call ssort(hat,xindex,nsct,2)
	    khat=1
            if(ainuin.eq.-999.)then
              ainu=0.
              ainu1=0.
            elseif((ainuin.gt.0.).and.(ainuin.lt.1.))then
              ainu=nsct*qbeta(ainuin,nref,nsct-nref)/nref
	      ainu1=xnsct*hat(nsct)/nref
	      khat=max(log(ainu1/ainu)/log(2.)+1.,1.)
            else
              ainu=abs(ainuin)
	      ainu1=xnsct*hat(nsct)/nref
	      khat=max(log(ainu1/ainu)/log(2.)+1.,1.)
            endif
            if(ainlin.eq.-999.)then
              ainl=0.
              ainl1=0.
            else
              do 156 k=1,nsct
  156           if(hat(k).ne.0.)goto 157
  157         ainl1=xnsct*hat(k)/nref
              if((ainlin.gt.0.).and.(ainlin.lt.1.))then
                ainl=nsct*qbeta(ainlin,nref,nsct-nref)/nref
              else
                ainl=abs(ainlin)
              endif
	      khat=max(float(khat),log(ainl/ainl1)/log(2.)+1.)
            endif
	    call ssort(xindex,hat,nsct,2)
	    write(nerr,*)' hat matrix iterations=',khat
            do 158 i=1,nsct
  158         what(i)=1.
            do 190 k=1,khat
c
c  begin hat matrix weight loop
c
              ainu1=max(ainu1/2.,ainu)
              ainl1=min(2.*ainl1,ainl)
              write(nerr,*)' lower cutoff=',ainl1,' upper cutoff=',ainu1
              do 159 i=1,nsct
                l1(i)=.false.
  159           l2(i)=.false.
              do 180 niter=1,10
c
c begin huber regression weight loop
c
                write(nerr,*)' huber iteration=',niter,' scale=',scale
                call hatwt(nsct,nref,ainl1,ainu1,l1,l2,hat,what)
                call huberwt(nsct,scale,res,w)
		do 160 i=1,nsct
  160             wt(i)=what(i)*w(i)
c
c  compute robust transfer function
c
                call transfer(nsct,nref,wt,ftsec,b,info)
                if(info.eq.1)then
                  write(nerr,*)
     $                  ' WARNING-singular matrix detected in',
     $                  '  huber magnetic transfer section'
                  write(nerr,*)' niter=',niter
                endif
c
c  compute residuals
c
                call residual(nsct,nref,ftsec,b,wt,nw,res,resq)
c
c  compute robust scale estimate
c
                nr=0
                do 176 i=1,nsct
                  if(wt(i).ne.0.)then
                    nr=nr+1
                    temp(nr)=res(i)
                  endif
  176             continue
                call mad(nr,temp,scale)
                if((ainlin.ne.-999.).or.(ainuin.ne.-999.))then
c
c  compute hat matrix diagonal
c
                  call hatdiag(nsct,nref,ftsec(1,2),wt,hat,info)
                  if(info.ne.0)then
                    write(nerr,*)
     $                ' WARNING-pivot block singular in huber magnetic',
     $                ' transfer call to hatdiag'
                    write(nerr,*)' info=',info,' niter=',niter
                  endif
                endif
                write(nerr,*)' normalized residual sum of squares=',
     $                       abs(resq-oresq)/abs(resq)
                if(abs(resq-oresq).lt.2.*eps*abs(resq))goto 185
  180           oresq=resq
              write(nerr,*)
     $              ' WARNING-huber iterations did not converge',
     $              ' at hat matrix iteration=',k
  185         oresq=resq
              write(nerr,*)' number of data=',nw
              write(nerr,*)
     $         ' huber transfer function indexed by reference variable'
              do 190 i=1,nref
  190           write(nerr,*)i,cmplx(b(i))
c
c  thomson weight regression section
c
            if(uin.ge.0.)then
              u0=sqrt(2.*log(xnsct/(uin+0.5)))
            else
              u0=abs(uin)
            endif
            do 240 niter=1,15
c
c  begin iteration on thomson regression weights
c
              write(nerr,*)
     $              ' thomson iteration=',niter,' scale=',scale
              call hatwt(nsct,nref,ainl,ainu,l1,l2,hat,what)
              call expwt(nsct,u0,scale,res,w)
              do 200 i=1,nsct
  200           wt(i)=what(i)*w(i)
c
c  compute robust transfer function
c
              call transfer(nsct,nref,wt,ftsec,b,info)
              if(info.eq.1)then
                write(nerr,*)
     $                ' WARNING-singular matrix detected in thomson',
     $                '  magnetic transfer section'
              endif
c
c  compute residuals
c
              call residual(nsct,nref,ftsec,b,wt,nw,res,resq)
              write(nerr,*)' number of data=',nw
              write(nerr,*)
     $         ' thomson solution vector indexed by reference variable'
              do 215 i=1,nref
  215           write(nerr,*)i,cmplx(b(i))
              write(nerr,*)' normalized residual sum of squares=',
     $                     abs(resq-oresq)/abs(resq)
c
c  compute robust scale estimate
c
              if(niter.le.2)then
                nr=0
                do 230 i=1,nsct
                  if(wt(i).ne.0.)then
                    nr=nr+1
                    temp(nr)=res(i)
                  endif
  230             continue
                call mad(nr,temp,scale)
              endif
              if((ainlin.ne.-999.).or.(ainuin.ne.-999.))then
                call hatdiag(nsct,nref,ftsec(1,2),wt,hat,info)
                if(info.ne.0)then
                  write(nerr,*)
     $                  ' WARNING-pivot block singular in',
     $                  ' thomson magnetic transfer call to hatdiag'
                  write(nerr,*)' info=',info,' niter=',niter
                endif
              endif
              if((niter.gt.1).and.
     $           (abs(resq-oresq).le.eps*abs(resq)))goto 250
  240         oresq=resq
	    do 241 i=1,nout
  241         convflag(i,nfl)=.false.
            write(nerr,*)' WARNING-thomson iterations did not converge'
  250       do 260 i=1,nref
  260         tb(i,nn,nfl)=b(i)
	    do 261 i=1,nsct
  261         wt2(i)=wt2(i)*wt(i)
            m1=0
            m2=0
            do 262 i=1,nsct
              if(l1(i))m1=m1+1
  262         if(l2(i))m2=m2+1
            write(nerr,*)m1,' lower and ',m2,' upper data deleted'
  265       continue
c
c  compute projected local magnetic field
c
  270     do 280 k=1,nsect
            wt1(k)=0.
	    do 275 j=1,ninp
              b(j)=0.
              do 275 i=1,nref
  275           b(j)=b(j)+ffts(k,nout+ninp+i,nf)*tb(i,j,nfl)
              do 280 i=1,ninp
                ffts(k,nout+ninp+i,nf)=b(i)
  280           ffts(k,nout+i,nf)=ffts(k,nout+ninp+i,nf)
	  do 290 i=1,nsct
	    wt1(ifix(tag(i)))=wt2(i)
  290       continue
  300     continue
c
c  begin second stage
c
c coherence estimation by block averaging
c
          do 310 i=1,nsect/nc2blk
            c2flag1(i)=.true.
            do 310 j=1,nout
  310         c2e(i,j)=1.0
          if(nlev.ge.2)then
            n1=min(nout,2)
            if((nout.eq.3).and.(nz.gt.0))n1=3
            call bcoher2(nsect,n1,nout,ninp,1,nc2blk,c2threshe,
     $                   c2trime,ffts(1,1,nf),xindex,c2flag,
     $                   c2cut,c2e,info)
            if(info.ne.0)then
              write(nerr,*)
     $          ' WARNING-singular matrix detected in',
     $          ' stage 2 coherence threshold at nsc=',nsc,' nf=',nf
            endif
            if(nz.gt.0)then
              do 315 i=1,nsect/nc2blk
  315           c2flag1(i)=c2flag(i)
            endif
            c2cut1=c2cut
            if((nout.eq.3).and.(nz.eq.0))then
              call bcoher2(nsect,1,1,ninp,1,nc2blk,c2threshe1,
     $                     c2trime1,ffts(1,nout,nf),xindex,c2flag1,
     $                     c2cut1,c2e(1,nout),info)
              if(info.ne.0)then
                write(nerr,*)
     $            ' WARNING-singular matrix detected in',
     $            ' stage 2 coherence threshold at nsc=',nsc,' nf=',nf
              endif
            endif
          endif
          write(nerr,*)' stage 2 coherence threshold=',c2cut
          if(nout.eq.3)
     $       write(nerr,*)' stage 2 3rd output coherence threshold=',
     $                    c2cut1
c
c  begin third loop pivoting on output variable
c
          do 770 nn=1,nout
            write(nerr,*)' #####################################'
            write(nerr,*)' output series=',nn
c
c  perform coherence thresholding as required
c
            if(nn.le.2)then
              call threshold(nsect,nc2blk,nout-nn+1,2*ninp,
     $                       c2flag,wt1,wtthresh,ffts(1,nn,nf),
     $                       nsct,tag,pwro,pwri,ftsec)
            elseif(nz.eq.0)then
              call threshold(nsect,nc2blk,1,2*ninp,c2flag1,wt1,
     $                       wtthresh,ffts(1,3,nf),nsct,tag,pwro,
     $                       pwri,ftsec)
            else
              call threshold(nsect,nc2blk,1,2*ninp,c2flag,wt1,
     $                       wtthresh,ffts(1,3,nf),nsct,tag,pwro,
     $                       pwri,ftsec)
            endif
            if((c2cut.gt.0.).or.(c2cut1.gt.0.))
     $        write(nerr,*)nsect-nsct,' sections discarded'
            write(nerr,*)' initial number of data=',nsct
	    fracthresh(nn,nfl)=float(nsct)/xnsect
            if(nsct.eq.0)goto 770
            xnsct=nsct
c
c  compute nonrobust transfer function
c
            do 320 i=1,nsct
  320         wt(i)=1.
            call response(nsct,ninp,wt,ftsec,b,aa,bb,info)
            if(info.eq.1)then
              write(nerr,*)' WARNING-singular matrix detected in',
     $                     ' nonrobust response section '
            endif
            write(nerr,*)
     $            ' initial solution vector indexed by input variable'
            do 322 i=1,ninp
              hn(nn,i,nfl)=b(i)
  322         write(nerr,*)i,hn(nn,i,nfl)
c
c  compute jackknife estimate of transfer function standard error
c  compute hat matrix diagonal
c
            call hatdiag(nsct,ninp,ftsec(1,ninp+2),wt,hat,info)
            if(info.ne.0)then
              write(nerr,*)
     $              ' WARNING-pivot block singular in nonrobust',
     $              ' response call to hatdiag  info=',info
            endif
c
c  compute delete one solutions for transfer functions
c
            call del1(nsct,ninp,ftsec,wt,aa,bb,hd1,info)
            if(info.ne.0)then
              write(nerr,*)
     $              ' WARNING-singular matrix detected in',
     $              ' nonrobust jackknife'
            endif
            if((nlev.ge.1).and.(rfout(1).eq.'y'))then
              write(22+nn)freq(nfl),nsct
              write(22+nn)(hat(i),(hd1(j,i),j=1,ninp),i=1,nsct)
            endif
c
c  form jackknife variance
c
            do 330 ii=1,ninp
              var=0.
              do 328 i=1,nsct
  328           var=var+abs((1.-hat(i))*(hn(nn,ii,nfl)-hd1(ii,i)))**2
  330         varn(nn,ii,nfl)=xnsct*var/(xnsct-ninp)
c
c  compute residuals
c
            call residual(nsct,ninp,ftsec,b,wt,nw,res,oresq)
c
c  compute nonrobust complex coherence between measured and predicted output
c  variables
c
            call ccoher(nsct,ninp,ftsec,hn(nn,1,nfl),hd1,wt,gd1,
     $                  gn(nn,nfl),gnlo(nn,nfl),gnhi(nn,nfl))
            dfn(nn,nfl)=2.*dofscal*(nsct-ninp)
c
c  compute initial robust scale estimate using nonrobust residuals
c
            do 350 i=1,nsct
  350         temp(i)=res(i)
            call mad(nsct,temp,scale)
c
c output nonrobust diagnostics if requested
c
            if(nlev.ge.3)then
              if(nfl.le.9)then
                cn(1:1)=char(48)
                cn(2:2)=char(48+nfl)
              else
                write(cn,997)nfl
              endif
              filnam=ofil(1:lenb(ofil))//'.'//char(48+nn)//
     $               '.oq.'//cn(1:2)
              open (unit=40,file=filnam,status='unknown')
              do 360 i=1,nsct
                xindex(i)=tag(i)
                qq(i)=res(i)
                yindex(i)=tag(i)
  360           temp(i)=xnsct*hat(i)/ninp
c  scale regression residuals
c  set second moment to 2 as expected for a rayleigh-distributed variate
              var=0.
              do 365 i=1,nsct
  365           var=var+qq(i)**2
              var=sqrt(2.*xnsct/var)
              do 367 i=1,nsct
  367           qq(i)=var*qq(i)
              call ssort(qq,xindex,nsct,2)
              call ssort(temp,yindex,nsct,2)
              write(40,999)freq(nfl)
c  output 10% of values at extremes and fraction of values in between
              do 370 i=1,ifix(0.1*xnsct)
                p=(float(i)-.5)/xnsct
  370           write(40,999)sqrt(2.*log(1./(1.-p))),qq(i),
     $                       nsct*qbeta(p,ninp,nsct-ninp)/ninp,
     $                       temp(i)
              do 372 i=ifix(0.1*xnsct)+1,ifix(0.9*xnsct),
     $                 max(nint(log10(xnsct)**1.5),1)
                p=(float(i)-.5)/xnsct
  372           write(40,999)sqrt(2.*log(1./(1.-p))),qq(i),
     $                       nsct*qbeta(p,ninp,nsct-ninp)/ninp,
     $                       temp(i)
              do 374 i=ifix(0.9*xnsct),nsct
                p=(float(i)-.5)/xnsct
  374           write(40,999)sqrt(2.*log(1./(1.-p))),qq(i),
     $                       nsct*qbeta(p,ninp,nsct-ninp)/ninp,
     $                       temp(i)
              close (unit=40)
            endif
            if(nlev.eq.4)then
              if(nfl.le.9)then
                cn(1:1)=char(48)
                cn(2:2)=char(48+nfl)
              else
                write(cn,997)nfl
              endif
              filnam=ofil(1:lenb(ofil))//'.'//char(48+nn)//'.or.'//
     $               cn(1:2)
              open (unit=40,file=filnam,status='unknown')
              respm=0.
              pwrom=0.
              pwrim=0.
              do 380 i=1,nsct
                respm=max(respm,abs(res(i))**2)
                pwrom=max(pwrom,pwro(i))
  380           pwrim=max(pwrim,pwri(i))
              write(40,999)freq(nfl),respm,pwrom,pwrim
              do 385 i=1,nsct
                write(40,999)xnsct*hat(i)/ninp,abs(res(i))**2/respm,
     $                       pwro(i)/pwrom,pwri(i)/pwrim
  385           continue
              close(unit=40)
            endif
c
c begin robust transfer function computation
c
            do 390 i=1,nsct
  390         xindex(i)=i
            if(uin.eq.-999.)goto 760
c
c  huber weight regression section
c
c
c  compute number of hat matrix iterations
c
            call ssort(hat,xindex,nsct,2)
            khat=1
            if(ainuin.eq.-999.)then
              ainu=0.
              ainu1=0.
            elseif((ainuin.gt.0.).and.(ainuin.lt.1.))then
              ainu=nsct*qbeta(ainuin,ninp,nsct-ninp)/ninp
	      ainu1=xnsct*hat(nsct)/ninp
	      khat=max(log(ainu1/ainu)/log(2.)+1.,1.)
            else
              ainu=abs(ainuin)
	      ainu1=xnsct*hat(nsct)/ninp
	      khat=max(log(ainu1/ainu)/log(2.)+1.,1.)
            endif
            if(ainlin.eq.-999.)then
              ainl=0.
              ainl1=0.
            else
              do 395 k=1,nsct
  395           if(hat(k).ne.0.)goto 396
  396         ainl1=xnsct*hat(k)/nref
              if((ainlin.gt.0.).and.(ainlin.lt.1.))then
                ainl=nsct*qbeta(ainlin,nref,nsct-nref)/nref
              else
                ainl=abs(ainlin)
              endif
	      khat=max(float(khat),log(ainl/ainl1)/log(2.)+1.)
            endif
            call ssort(xindex,hat,nsct,2)
            write(nerr,*)' hat matrix iterations=',khat
            do 400 i=1,nsct
              l1(i)=.false.
              l2(i)=.false.
  400         what(i)=1.
            do 460 k=1,khat
c
c  begin hat matrix weight loop
c
              ainu1=max(ainu1/2.,ainu)
              ainl1=min(2.*ainl1,ainl)
              write(nerr,*)' lower cutoff=',ainl1,' upper cutoff=',ainu1
              do 450 niter=1,10
c
c begin huber regression weight loop
c
                write(nerr,*)' huber iteration=',niter,' scale=',scale
                call hatwt(nsct,ninp,ainl1,ainu1,l1,l2,hat,what)
                call huberwt(nsct,scale,res,w)
                do 410 i=1,nsct
  410             wt(i)=what(i)*w(i)
c
c  compute robust transfer function
c
                call response(nsct,ninp,wt,ftsec,b,aa,bb,info)
                if(info.eq.1)then
                  write(nerr,*)
     $                  ' WARNING-singular matrix detected in',
     $                  '  huber section'
                  write(nerr,*)' niter=',niter
                endif
c
c  compute residuals
c
                call residual(nsct,ninp,ftsec,b,wt,nw,res,resq)
c
c  compute robust scale estimate
c
                nr=0
                do 440 i=1,nsct
                  if(wt(i).ne.0.)then
                    nr=nr+1
                    temp(nr)=res(i)
                  endif
  440             continue
                call mad(nr,temp,scale)
                if((ainlin.ne.-999.).or.(ainuin.ne.-999.))then
c
c  compute hat matrix diagonal
c
                  call hatdiag(nsct,ninp,ftsec(1,ninp+2),
     $                         wt,hat,info)
                  if(info.ne.0)then
                    write(nerr,*)
     $                ' WARNING-pivot block singular in huber call',
     $                ' to hatdiag'
                    write(nerr,*)' info=',info,' niter=',niter
                  endif
                endif
                write(nerr,*)' normalized residual sum of squares=',
     $                       abs(resq-oresq)/abs(resq)
                if(abs(resq-oresq).lt.2.*eps*abs(resq))
     $             goto 455
  450           oresq=resq
              write(nerr,*)
     $              ' WARNING-huber iterations did not converge',
     $              ' at hat matrix iteration=',k
  455         oresq=resq
              write(nerr,*)' number of data=',nw
              write(nerr,*)
     $              ' huber solution vector indexed by input variable'
              do 460 i=1,ninp
                ob(i)=b(i)
  460           write(nerr,*)i,cmplx(b(i))
c
c  thomson weight regression section
c
            if(uin.ge.0)then
              u0=sqrt(2.*log(xnsct/(uin+0.5)))
            else
              u0=abs(uin)
            endif
            call hatwt(nsct,ninp,ainl,ainu,l1,l2,hat,what)
            call expwt(nsct,u0,scale,res,w)
            do 465 i=1,nsct
  465           wt(i)=what(i)*w(i)
            do 500 niter=1,15
c
c  begin iteration on thomson regression weights
c
              write(nerr,*)
     $              ' thomson iteration=',niter,' scale=',scale
c
c  compute robust transfer function
c
              call response(nsct,ninp,wt,ftsec,b,aa,bb,info)
              if(info.eq.1)then
                write(nerr,*)
     $                ' WARNING-singular matrix detected in thomson',
     $                '  section'
              endif
c
c  compute residuals
c
              call residual(nsct,ninp,ftsec,b,wt,nw,res,resq)
              write(nerr,*)' number of data=',nw
              write(nerr,*)
     $              ' thomson solution vector indexed by input variable'
              do 470 i=1,ninp
                hr(nn,i,nfl)=b(i)
  470           write(nerr,*)i,cmplx(b(i))
              write(nerr,*)' normalized residual sum of squares=',
     $                     abs(resq-oresq)/abs(resq)
c
c  compute robust scale estimate
c
              if(niter.le.2)then
                nr=0
                do 475 i=1,nsct
                  if(wt(i).ne.0.)then
                    nr=nr+1
                    temp(nr)=res(i)
                  endif
  475             continue
                call mad(nr,temp,scale)
              endif
c
c  compute hat matrix diagonal
c
              call hatdiag(nsct,ninp,ftsec(1,ninp+2),wt,hat,info)
              if(info.ne.0)then
                write(nerr,*)
     $                ' WARNING-pivot block singular in',
     $                ' thomson call to hatdiag'
                write(nerr,*)' info=',info,' niter=',niter
              endif
              call hatwt(nsct,ninp,ainl,ainu,l1,l2,hat,what)
	      call expwt(nsct,u0,scale,res,w)
	      do 480 i=1,nsct
  480           wt(i)=what(i)*w(i)
c
c  compute jackknife estimate of transfer function standard error
c
c
c  compute delete one solutions for transfer functions
c
              call del1(nsct,ninp,ftsec,wt,aa,bb,hd1,info)
	      if(info.ne.0)then
	        write(nerr,*)
     $              ' WARNING-singular matrix detected in robust',
     $              ' jackknife niter=',niter
              endif
c
c  form jackknife variance excluding values with zero weights
c
              do 490 ii=1,ninp
	        var=0.
	        nw=0
	        do 485 i=1,nsct
	          if(wt(i).ne.0.)then
	            nw=nw+1
		    var=var+abs((1.-hat(i))*
     $                      (hr(nn,ii,nfl)-hd1(ii,i)))**2
                  endif
  485         continue
  490         varr(nn,ii,nfl)=nw*var/(nw-ninp)
	      cflag1=.true.
	      do 495 i=1,ninp
		sdev=sqrt(varr(nn,i,nfl))
		if((abs(dreal(ob(i)-b(i))).ge.eps1*sdev).or.
     $             (abs(dimag(ob(i)-b(i))).ge.eps1*sdev))
     $             cflag1=.false.
  495           ob(i)=b(i)
		cflag2=.false.
		if((abs(real(resq-oresq)).lt.eps*real(resq)).and.
     $             (abs(aimag(resq-oresq)).lt.eps*aimag(resq)))
     $             cflag2=.true.
	      if((niter.gt.1).and.(cflag1.or.cflag2))goto 510
  500         oresq=resq
	    convflag(nn,nfl)=.false.
	    write(nerr,*)' WARNING-thomson iterations did not converge'
  510       if(cflag1)write(nerr,*)' convergence on response'
	    if(cflag2)write(nerr,*)' convergence on residuals'
            m1=0
            m2=0
            do 511 i=1,nsct
              if(l1(i))m1=m1+1
  511         if(l2(i))m2=m2+1
            write(nerr,*)m1,' lower and ',m2,' upper data deleted'
c
c  compute complex coherence between measured and predicted output variables
c
	    call ccoher(nsct,ninp,ftsec,hr(nn,1,nfl),hd1,wt,gd1,
     $                  gr(nn,nfl),grlo(nn,nfl),grhi(nn,nfl))
	    wtsum=0.
	    do 540 i=1,nsct
  540         wtsum=wtsum+wt(i)
	    dfr(nn,nfl)=2.*dofscal*(wtsum-ninp)
	    fracdat(nn,nfl)=fracthresh(nn,nfl)*wtsum/xnsct
	    if((fracdat(nn,nfl).ne.0.).and.(nlev.ge.1).and.
     $         (rfout(2).eq.'y'))then
	      write(32+nn)freq(nfl),nsct
	      write(32+nn)(hat(i),wt(i),(hd1(j,i),j=1,ninp),
     $                     i=1,nsct)
	    endif
c
c  output robust diagnostics as required
c
	    if(nlev.ge.3)then
	      if(nfl.le.9)then
		cn(1:1)=char(48)
		cn(2:2)=char(48+nfl)
	      else
		write(cn,997)nfl
	      endif
	      filnam=ofil(1:lenb(ofil))//'.'//char(48+nn)//
     $               '.fq.'//cn(1:2)
	      open (unit=40,file=filnam,status='unknown')
	      var=0.
	      do 550 i=1,nsct
                WT1(I)=1.
		temp(i)=(wt(i)*res(i))**2
  550           var=var+temp(i)
c  scale regression residuals
c  set second moment to 2 as expected for rayleigh-distributed variate
	      var=sqrt(2.*nw/var)
	      nq=0
	      do 555 i=1,nsct
		if(wt(i).gt.0.)then
		  nq=nq+1
		  xindex(nq)=tag(i)
		  qq(nq)=var*sqrt(temp(i))
		  yindex(nq)=tag(i)
		  temp(nq)=nw*hat(i)/ninp
		endif
  555           continue
	      call ssort(qq,xindex,nq,2)
	      call ssort(temp,yindex,nq,2)
	      xnw=nw
	      write(40,999)freq(nfl),xnw,xnsct
              t1=sqrt(2.*log(xnsct/(xnsct-xnw+.5)))
              ft1=1-exp(-t1*t1/2.)
              t2=0.
              ft2=0.
              if(m1.gt.0)then
                t2=qbeta((float(m1)-.5)/xnsct,ninp,nsct-ninp)
                ft2=betai(t2,float(ninp),xnsct-float(ninp))
              endif
              t3=qbeta((float(nsct-m2)-.5)/xnsct,ninp,nsct-ninp)
              ft3=betai(t3,float(ninp),xnsct-float(ninp))-ft2
              do 560 i=1,ifix(0.1*xnw)
		p=(float(i)-.5)/xnw
  560           write(40,999)sqrt(2.*log(1./(1.-ft1*p))),qq(i),
     $                       xnw*qbeta(ft3*p+ft2,ninp,nsct-ninp)/ninp,
     $                       temp(i)
              do 562 i=ifix(0.1*xnw)+1,ifix(0.9*xnw),
     $                 max(nint(log10(xnw)**1.5),1)
               p=(float(i)-.5)/xnw
  562           write(40,999)sqrt(2.*log(1./(1.-ft1*p))),qq(i),
     $                       xnw*qbeta(ft3*p+ft2,ninp,nsct-ninp)/ninp,
     $                       temp(i)
              do 564 i=ifix(0.9*xnw),nw
                p=(float(i)-.5)/xnw
  564           write(40,999)sqrt(2.*log(1./(1.-ft1*p))),qq(i),
     $                       xnw*qbeta(ft3*p+ft2,ninp,nsct-ninp)/ninp,
     $                       temp(i)
	      close (unit=40)
	    endif
	    if(nlev.eq.4)then
	      if(nfl.le.9)then
		cn(1:1)=char(48)
		cn(2:2)=char(48+nfl)
	      else
		write(cn,997)nfl
	      endif
	      filnam=ofil(1:lenb(ofil))//'.'//char(48+nn)//'.fr.'//
     $               cn(1:2)
	      open (unit=40,file=filnam,status='unknown')
	      respm=0.
	      pwrom=0.
	      pwrim=0.
	      do 570 i=1,nsct
		respm=max(respm,(wt(i)*abs(res(i)))**2)
		pwrom=max(pwrom,wt(i)**2*pwro(i))
  570           pwrim=max(pwrim,wt(i)**2*pwri(i))
	      write(40,999)freq(nfl),respm,pwrom,pwrim
	      do 575 i=1,nsct
		if(wt(i).ne.0.)then
		  write(40,999)nq*hat(i)/ninp,
     $                         (wt(i)*abs(res(i)))**2/respm,
     $                         wt(i)**2*pwro(i)/pwrom,
     $                         wt(i)**2*pwri(i)/pwrim
		endif
  575           continue
	      close(unit=40)
	    endif
  760       if(nlev.ge.2)then
	      if(nfl.le.9)then
		cn(1:1)=char(48)
		cn(2:2)=char(48+nfl)
	      else
		write(cn,997)nfl
	      endif
	      filnam=ofil(1:lenb(ofil))//'.'//char(48+nn)//
     $               '.wt.'//cn(1:2)
	      open (unit=40,file=filnam,status='unknown')
	      k=1
	      if(nrr.eq.0)then
		do 764 ii=1,nsect/nc2blk
		  if(((nn.le.2).and.(c2flag(ii))).or.
     $               ((nn.eq.3).and.(nz.gt.0).and.(c2flag(ii))).or.
     $               ((nn.eq.3).and.(nz.eq.0).and.(c2flag1(ii))))then
		    do 762 i=1,nc2blk
		      write(40,999)tag(k)*xnfft/2.,c2e(ii,nn),
     $                             w(k),what(k)
		      k=k+1
  762                 continue
		  else
		    do 763 i=1,nc2blk
  763                 write(40,999)float((ii-1)*nc2blk+i)*xnfft/2.,
     $                             c2e(ii,nn),0.,0.
		  endif
  764             continue
	      else
		do 768 ii=1,nsect/nc2blk
		  if(((nn.le.2).and.(c2flag(ii))).or.
     $               ((nn.eq.3).and.(nz.gt.0).and.(c2flag(ii))).or.
     $               ((nn.eq.3).and.(nz.eq.0).and.(c2flag1(ii))))then
		    do 766 i=1,nc2blk
		      if(wt1((ii-1)*nc2blk+i).gt.wtthresh)then
			write(40,999)tag(k)*xnfft/2.,c2e(ii,nn),
     $                               (c2b(ii,j),j=1,ninp),
     $                               wt1((ii-1)*nc2blk+i),w(k),what(k)
			k=k+1
		      else
			write(40,999)float((ii-1)*nc2blk+i)*xnfft/2.,
     $                               c2e(ii,nn),(c2b(ii,j),j=1,ninp),
     $                               wt1((ii-1)*nc2blk+i),0.,0.
		      endif
  766                 continue
		  else
		    do 767 i=1,nc2blk
  767                 write(40,999)float((ii-1)*nc2blk+i)*xnfft/2.,
     $                             c2e(ii,nn),(c2b(ii,j),j=1,ninp),
     $                             wt1((ii-1)*nc2blk+i),0.,0.
		  endif
  768             continue
	      endif
	      close (unit=40)
	    endif
  770       continue
	  nfl=nfl+1
  780     continue
	nfft=nfft/nsctinc
  790   continue
c
c  output section
c
      do 902 n=1,nout
	if(rfout(1).eq.'y')close(unit=22+n)
	if(rfout(2).eq.'y')close(unit=32+n)
  902   continue
      do 950 n=1,nout
	do 928 j=1,ninp
	  postfix='.'//char(48+n)//'n'//char(48+j)//'.rf'
	  filnam=ofil(1:lenb(ofil))//postfix(1:lenb(postfix))
	  open (unit=10,file=filnam,status='unknown')
	  do 910 i=1,nfl-1
	    if(fracthresh(n,i).ne.0.)then
	      write(10,999)freq(i),real(hn(n,j,i)),aimag(hn(n,j,i)),
     $                     sqrt(varn(n,j,i)),dfn(n,i),fracthresh(n,i)
	    endif
  910       continue
	  close (unit=10)
	  if(uin.ne.-999.)then
	    postfix='.'//char(48+n)//'r'//char(48+j)//'.rf'
	    filnam=ofil(1:lenb(ofil))//postfix(1:lenb(postfix))
	    open (unit=10,file=filnam,status='unknown')
	    do 920 i=1,nfl-1
	      if(fracdat(n,i).ne.0.)then
		if(.not.convflag(n,i))then
		  write(10,1000)freq(i),real(hr(n,j,i)),
     $                          aimag(hr(n,j,i)),sqrt(varr(n,j,i)),
     $                          dfr(n,i),fracthresh(n,i),
     $                          fracdat(n,i),star
		else
		  write(10,999)freq(i),real(hr(n,j,i)),
     $                         aimag(hr(n,j,i)),sqrt(varr(n,j,i)),
     $                         dfr(n,i),fracthresh(n,i),fracdat(n,i)
		endif
	      endif
  920         continue
	    close (unit=10)
	  endif
  928     continue
	postfix='.'//char(48+n)//'n.c2'
	filnam=ofil(1:lenb(ofil))//postfix(1:lenb(postfix))
	open (unit=10,file=filnam,status='unknown')
	do 930 i=1,nfl-1
	  if(fracthresh(n,i).ne.0.)then
	    write(10,999)freq(i),abs(gn(n,i))**2,phase(gn(n,i)),
     $                   1.-.05**(2./(dfn(n,i)-2.)),
     $                   gnlo(n,i)**2,gnhi(n,i)**2,dfn(n,i),
     $                   fracthresh(n,i)
	  endif
  930     continue
	close (unit=10)
	if(uin.ne.-999.)then
	  postfix='.'//char(48+n)//'r.c2'
	  filnam=ofil(1:lenb(ofil))//postfix(1:lenb(postfix))
	  open (unit=10,file=filnam,status='unknown')
	  do 940 i=1,nfl-1
	    if(fracdat(n,i).ne.0.)then
	      write(10,999)freq(i),abs(gr(n,i))**2,phase(gr(n,i)),
     $                     1.-.05**(2./(dfr(n,i)-2.)),
     $                     grlo(n,i)**2,grhi(n,i)**2,dfr(n,i),
     $                     fracthresh(n,i),fracdat(n,i)
	    endif
  940       continue
	  close (unit=10)
	endif
  950   continue
      if(nrr.eq.1)then
	do 970 j=1,ninp
	  postfix='.'//char(48+j)//'.tb'
	  filnam=ofil(1:lenb(ofil))//postfix(1:lenb(postfix))
	  open (unit=10,file=filnam,status='unknown')
	  do 960 i=1,nfl-1
	    if(fracthresh(1,i).ne.0.)then
	      write(10,999)freq(i),(real(tb(k,j,i)),
     $                              aimag(tb(k,j,i)),k=1,nref)
	    endif
  960       continue
  970     close (unit=10)
      endif
  997 format(i2)
  999 format(x,11g15.7)
 1000 format(x,7g15.7,a)
      end
