      SUBROUTINE SFT(X,N,OM,CT,ST)
C  CALCULATES FOURIER TRANSFORM OF REAL SEQUENCE X(I),I=1,...N
C  AT ANGULAR FREQUENCY OM NORMALIZED SO THAT NYQUIST=PI. THE SINE
C  TRANSFORM IS RETURNED IN ST AND THE COSINE TRANSFORM IN CT.
C  ALGORITHM IS THAT OF GOERTZAL WITH MODIFICATIONS BY
C  GENTLEMAN, COMP.J. 1969
C  TRANSFORM IS NOT NORMALIZED
C  TO NORMALIZE ONE-SIDED FT, DIVIDE BY SQRT(DATA LENGTH)
C  FOR POSITIVE OM, THE FT IS DEFINED AS CT+(0.,1.)ST OR LIKE SLATEC
C  CFFTF
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (PI=3.141592653589793238D0,TP=2.D0*PI)
      DIMENSION X(N)
      NP1=N+1
      L=6.D0*OM/TP
      S=SIN(OM)
      A=0.D0
      C=0.D0
      D=0.D0
      E=0.D0
      IF(L.EQ.0)THEN
C  RECURSION FOR LOW FREQUENCIES (.LT. NYQ/3)
        S2=SIN(OM/2.D0)
        B=-4.D0*S2*S2
        DO 10 K=1,N
          C=A
          D=E
          A=X(NP1-K)+B*D+C
   10     E=A+D
      ELSEIF(L.EQ.1)THEN
C  REGULAR GOERTZAL ALGORITHM FOR INTERMEDIATE FREQUENCIES
        B=2.D0*COS(OM)
        DO 20 K=1,N
          A=X(NP1-K)+B*E-D
          D=E
   20     E=A
      ELSE
C  RECURSION FOR HIGH FREQUENCIES (.GT. 2*NYQ/3)
        C2=COS(OM/2.D0)
        B=4.D0*C2*C2
        DO 30 K=1,N
          C=A
          D=E
          A=X(NP1-K)+B*D-C
   30     E=A-D
      ENDIF
      ST=-S*D
      CT=A-B*D/2.D0
      RETURN
      END
